@@ -1,4 +1,5 @@
 use arc_swap::{ArcSwap, ArcSwapOption};
+use paste::paste;
 use std::sync::{
     atomic::{AtomicUsize, Ordering as AtomicOrdering},
     Arc,
@@ -16,42 +17,43 @@ use crate::{Message, Source};
 ///
 /// # Examples
 ///
-/// ```
-/// use arc_swap::ArcSwap;
+/// ```no_run
+/// use async_executors::TimerExt;
 /// use async_nursery::Nursery;
+/// use crossbeam_queue::SegQueue;
 /// use std::{sync::Arc, time::Duration};
 ///
 /// use callbag::{combine, for_each, interval};
 ///
 /// let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
 ///
-/// let vec = Arc::new(ArcSwap::from_pointee(vec![]));
+/// let actual = Arc::new(SegQueue::new());
 ///
 /// let source = combine!(
-///     interval(Duration::from_millis(100), nursery.clone()).into(),
-///     interval(Duration::from_millis(350), nursery.clone()).into(),
+///     interval(Duration::from_millis(100), nursery.clone()),
+///     interval(Duration::from_millis(350), nursery.clone()),
 /// );
 ///
 /// for_each({
-///     let vec = Arc::clone(&vec);
+///     let actual = Arc::clone(&actual);
 ///     move |x| {
 ///         println!("{:?}", x);
-///         vec.rcu(move |vec| {
-///             let mut vec = (**vec).clone();
-///             vec.push(x);
-///             vec
-///         });
+///         actual.push(x);
 ///     }
 /// })(source);
 ///
+/// let nursery_out = nursery.timeout(Duration::from_millis(1_000), nursery_out);
 /// drop(nursery);
-/// async_std::task::block_on(async_std::future::timeout(
-///     Duration::from_millis(1_000),
-///     nursery_out,
-/// ));
+/// async_std::task::block_on(nursery_out);
 ///
 /// assert_eq!(
-///     vec.load()[..],
+///     &{
+///         let mut v = vec![];
+///         for _i in 0..actual.len() {
+///             v.push(actual.pop().unwrap());
+///         }
+///         v
+///     }[..],
 ///     [
 ///         (2, 0),
 ///         (3, 0),
@@ -81,7 +83,7 @@ macro_rules! combine_impls {
         $Combine:ident {
             $(($idx:tt) -> $T:ident)+
         }
-    )+) => {
+    )+) => { paste! {
         $(
             impl<$($T),+> Unwrap for ($(Option<$T>,)+) {
                 type Output = ($($T,)+);
@@ -91,10 +93,19 @@ macro_rules! combine_impls {
                 }
             }
 
-            impl<$($T: Send + Sync + Clone + 'static),+> Combine for ($(Arc<Source<$T>>,)+) {
+            impl<$($T: 'static, [<S $T>]: 'static),+> Combine for ($([<S $T>],)+)
+            where
+                $(
+                    $T: Send + Sync + Clone,
+                    [<S $T>]: IntoArcSource<Output = $T> + Send + Sync,
+                )+
+            {
                 type Output = ($($T,)+);
 
                 fn combine(self) -> Source<Self::Output> {
+                    $(
+                        let [<source_ $idx>] = self.$idx.into_arc_source();
+                    )+
                     (move |message| {
                         if let Message::Handshake(sink) = message {
                             const N: usize = last_literal!($($idx,)+) + 1;
@@ -116,21 +127,21 @@ macro_rules! combine_impls {
                                         Message::Pull => {
                                             $({
                                                 let source_talkback = source_talkbacks.$idx.load();
-                                                let source_talkback = source_talkback.as_ref().unwrap();
+                                                let source_talkback = source_talkback.as_ref().expect("source talkback not set");
                                                 source_talkback(Message::Pull);
                                             })+
                                         }
                                         Message::Error(ref error) => {
                                             $({
                                                 let source_talkback = source_talkbacks.$idx.load();
-                                                let source_talkback = source_talkback.as_ref().unwrap();
+                                                let source_talkback = source_talkback.as_ref().expect("source talkback not set");
                                                 source_talkback(Message::Error(error.clone()));
                                             })+
                                         }
                                         Message::Terminate => {
                                             $({
                                                 let source_talkback = source_talkbacks.$idx.load();
-                                                let source_talkback = source_talkback.as_ref().unwrap();
+                                                let source_talkback = source_talkback.as_ref().expect("source talkback not set");
                                                 source_talkback(Message::Terminate);
                                             })+
                                         }
@@ -139,7 +150,7 @@ macro_rules! combine_impls {
                                 .into()
                             );
                             $(
-                                (self.$idx)(Message::Handshake(Arc::new(
+                                [<source_ $idx>](Message::Handshake(Arc::new(
                                     {
                                         let sink = Arc::clone(&sink);
                                         let n_start = Arc::clone(&n_start);
@@ -191,7 +202,7 @@ macro_rules! combine_impls {
                 }
             }
         )+
-    };
+    } };
 }
 
 macro_rules! last_literal {
@@ -310,12 +321,42 @@ pub trait Combine {
     fn combine(self) -> Source<Self::Output>;
 }
 
+pub trait IntoArcSource {
+    type Output;
+
+    fn into_arc_source(self) -> Arc<Source<Self::Output>>;
+}
+
 trait Unwrap {
     type Output;
 
     fn unwrap(self) -> Self::Output;
 }
 
+impl<T> IntoArcSource for Arc<Source<T>> {
+    type Output = T;
+
+    fn into_arc_source(self) -> Arc<Source<Self::Output>> {
+        self
+    }
+}
+
+impl<T> IntoArcSource for Source<T> {
+    type Output = T;
+
+    fn into_arc_source(self) -> Arc<Source<Self::Output>> {
+        Arc::new(self)
+    }
+}
+
+impl<T> IntoArcSource for Box<Source<T>> {
+    type Output = T;
+
+    fn into_arc_source(self) -> Arc<Source<Self::Output>> {
+        Arc::from(self)
+    }
+}
+
 /// Callbag factory that combines the latest data points from multiple (2 or more) callbag sources.
 ///
 /// It delivers those latest values as a tuple.
