@@ -12,28 +12,33 @@ use crate::{Message, Source};
 /// # Examples
 ///
 /// ```
-/// use arc_swap::ArcSwap;
+/// use crossbeam_queue::SegQueue;
 /// use std::sync::Arc;
 ///
 /// use callbag::{filter, for_each, from_iter};
 ///
-/// let vec = Arc::new(ArcSwap::from_pointee(vec![]));
+/// let actual = Arc::new(SegQueue::new());
 ///
 /// let source = filter(|x| x % 2 == 1)(from_iter([1, 2, 3, 4, 5]));
 ///
 /// for_each({
-///     let vec = Arc::clone(&vec);
+///     let actual = Arc::clone(&actual);
 ///     move |x| {
 ///         println!("{}", x);
-///         vec.rcu(move |vec| {
-///             let mut vec = (**vec).clone();
-///             vec.push(x);
-///             vec
-///         });
+///         actual.push(x);
 ///     }
 /// })(source);
 ///
-/// assert_eq!(vec.load()[..], [1, 3, 5]);
+/// assert_eq!(
+///     &{
+///         let mut v = vec![];
+///         for _i in 0..actual.len() {
+///             v.push(actual.pop().unwrap());
+///         }
+///         v
+///     }[..],
+///     [1, 3, 5]
+/// );
 /// ```
 pub fn filter<I: 'static, F: 'static, S>(condition: F) -> Box<dyn Fn(S) -> Source<I>>
 where
@@ -59,47 +64,54 @@ where
                                             (move |message| match message {
                                                 Message::Handshake(_) => {
                                                     panic!("sink handshake has already occurred");
-                                                }
+                                                },
                                                 Message::Data(_) => {
                                                     panic!("sink must not send data");
-                                                }
+                                                },
                                                 Message::Pull => {
                                                     let talkback = talkback.load();
-                                                    let source = talkback.as_ref().unwrap();
+                                                    let source = talkback
+                                                        .as_ref()
+                                                        .expect("source talkback not set");
                                                     source(Message::Pull);
-                                                }
+                                                },
                                                 Message::Error(error) => {
                                                     let talkback = talkback.load();
-                                                    let source = talkback.as_ref().unwrap();
+                                                    let source = talkback
+                                                        .as_ref()
+                                                        .expect("source talkback not set");
                                                     source(Message::Error(error));
-                                                }
+                                                },
                                                 Message::Terminate => {
                                                     let talkback = talkback.load();
-                                                    let source = talkback.as_ref().unwrap();
+                                                    let source = talkback
+                                                        .as_ref()
+                                                        .expect("source talkback not set");
                                                     source(Message::Terminate);
-                                                }
+                                                },
                                             })
                                             .into(),
                                         )));
-                                    }
+                                    },
                                     Message::Data(data) => {
                                         if condition(&data) {
                                             sink(Message::Data(data));
                                         } else {
                                             let talkback = talkback.load();
-                                            let talkback = talkback.as_ref().unwrap();
+                                            let talkback =
+                                                talkback.as_ref().expect("source talkback not set");
                                             talkback(Message::Pull);
                                         }
-                                    }
+                                    },
                                     Message::Pull => {
                                         panic!("source must not pull");
-                                    }
+                                    },
                                     Message::Error(error) => {
                                         sink(Message::Error(error));
-                                    }
+                                    },
                                     Message::Terminate => {
                                         sink(Message::Terminate);
-                                    }
+                                    },
                                 }
                             }
                         }
