@@ -17,43 +17,47 @@ use crate::{Message, Source};
 /// On a listenable source:
 ///
 /// ```
-/// use arc_swap::ArcSwap;
+/// use async_executors::TimerExt;
 /// use async_nursery::Nursery;
+/// use crossbeam_queue::SegQueue;
 /// use std::{sync::Arc, time::Duration};
 ///
 /// use callbag::{for_each, interval, skip};
 ///
 /// let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
 ///
-/// let vec = Arc::new(ArcSwap::from_pointee(vec![]));
+/// let actual = Arc::new(SegQueue::new());
 ///
 /// let source = skip(3)(interval(Duration::from_millis(1_000), nursery.clone()));
 ///
 /// for_each({
-///     let vec = Arc::clone(&vec);
+///     let actual = Arc::clone(&actual);
 ///     move |x| {
 ///         println!("{}", x);
-///         vec.rcu(move |vec| {
-///             let mut vec = (**vec).clone();
-///             vec.push(x);
-///             vec
-///         });
+///         actual.push(x);
 ///     }
 /// })(source);
 ///
+/// let nursery_out = nursery.timeout(Duration::from_millis(7_500), nursery_out);
 /// drop(nursery);
-/// async_std::task::block_on(async_std::future::timeout(
-///     Duration::from_millis(7_500),
-///     nursery_out,
-/// ));
+/// async_std::task::block_on(nursery_out);
 ///
-/// assert_eq!(vec.load()[..], [3, 4, 5, 6]);
+/// assert_eq!(
+///     &{
+///         let mut v = vec![];
+///         for _i in 0..actual.len() {
+///             v.push(actual.pop().unwrap());
+///         }
+///         v
+///     }[..],
+///     [3, 4, 5, 6]
+/// );
 /// ```
 ///
 /// On a pullable source:
 ///
 /// ```
-/// use arc_swap::ArcSwap;
+/// use crossbeam_queue::SegQueue;
 /// use std::sync::Arc;
 ///
 /// use callbag::{for_each, from_iter, skip};
@@ -84,23 +88,28 @@ use crate::{Message, Source};
 ///     }
 /// }
 ///
-/// let vec = Arc::new(ArcSwap::from_pointee(vec![]));
+/// let actual = Arc::new(SegQueue::new());
 ///
 /// let source = skip(4)(from_iter(Range::new(10, 20)));
 ///
 /// for_each({
-///     let vec = Arc::clone(&vec);
+///     let actual = Arc::clone(&actual);
 ///     move |x| {
 ///         println!("{}", x);
-///         vec.rcu(move |vec| {
-///             let mut vec = (**vec).clone();
-///             vec.push(x);
-///             vec
-///         });
+///         actual.push(x);
 ///     }
 /// })(source);
 ///
-/// assert_eq!(vec.load()[..], [14, 15, 16, 17, 18, 19, 20]);
+/// assert_eq!(
+///     &{
+///         let mut v = vec![];
+///         for _i in 0..actual.len() {
+///             v.push(actual.pop().unwrap());
+///         }
+///         v
+///     }[..],
+///     [14, 15, 16, 17, 18, 19, 20]
+/// );
 /// ```
 pub fn skip<T: 'static, S>(max: usize) -> Box<dyn Fn(S) -> Source<T>>
 where
@@ -122,51 +131,55 @@ where
                                     move |message| match message {
                                         Message::Handshake(_) => {
                                             panic!("sink handshake has already occurred");
-                                        }
+                                        },
                                         Message::Data(_) => {
                                             panic!("sink must not send data");
-                                        }
+                                        },
                                         Message::Pull => {
                                             let talkback = talkback.load();
-                                            let source = talkback.as_ref().unwrap();
+                                            let source =
+                                                talkback.as_ref().expect("source talkback not set");
                                             source(Message::Pull);
-                                        }
+                                        },
                                         Message::Error(error) => {
                                             let talkback = talkback.load();
-                                            let source = talkback.as_ref().unwrap();
+                                            let source =
+                                                talkback.as_ref().expect("source talkback not set");
                                             source(Message::Error(error));
-                                        }
+                                        },
                                         Message::Terminate => {
                                             let talkback = talkback.load();
-                                            let source = talkback.as_ref().unwrap();
+                                            let source =
+                                                talkback.as_ref().expect("source talkback not set");
                                             source(Message::Terminate);
-                                        }
+                                        },
                                     }
                                 }
                                 .into(),
                             )));
-                        }
+                        },
                         Message::Data(data) => {
                             if skipped.load(AtomicOrdering::Acquire) < max {
                                 skipped.fetch_add(1, AtomicOrdering::AcqRel);
                                 {
                                     let talkback = talkback.load();
-                                    let talkback = talkback.as_ref().unwrap();
+                                    let talkback =
+                                        talkback.as_ref().expect("source talkback not set");
                                     talkback(Message::Pull);
                                 }
                             } else {
                                 sink(Message::Data(data));
                             }
-                        }
+                        },
                         Message::Pull => {
                             panic!("source must not pull");
-                        }
+                        },
                         Message::Error(error) => {
                             sink(Message::Error(error));
-                        }
+                        },
                         Message::Terminate => {
                             sink(Message::Terminate);
-                        }
+                        },
                     })
                     .into(),
                 )))
