@@ -17,40 +17,45 @@ use crate::{Message, Source};
 /// On a listenable source:
 ///
 /// ```
-/// use arc_swap::ArcSwap;
 /// use async_nursery::Nursery;
+/// use crossbeam_queue::SegQueue;
 /// use std::{sync::Arc, time::Duration};
 ///
 /// use callbag::{for_each, interval, take};
 ///
 /// let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
 ///
-/// let vec = Arc::new(ArcSwap::from_pointee(vec![]));
+/// let actual = Arc::new(SegQueue::new());
 ///
 /// let source = take(3)(interval(Duration::from_millis(1_000), nursery.clone()));
 ///
 /// for_each({
-///     let vec = Arc::clone(&vec);
+///     let actual = Arc::clone(&actual);
 ///     move |x| {
 ///         println!("{}", x);
-///         vec.rcu(move |vec| {
-///             let mut vec = (**vec).clone();
-///             vec.push(x);
-///             vec
-///         });
+///         actual.push(x);
 ///     }
 /// })(source);
 ///
 /// drop(nursery);
 /// async_std::task::block_on(nursery_out);
 ///
-/// assert_eq!(vec.load()[..], [0, 1, 2]);
+/// assert_eq!(
+///     &{
+///         let mut v = vec![];
+///         for _i in 0..actual.len() {
+///             v.push(actual.pop().unwrap());
+///         }
+///         v
+///     }[..],
+///     [0, 1, 2]
+/// );
 /// ```
 ///
 /// On a pullable source:
 ///
 /// ```
-/// use arc_swap::ArcSwap;
+/// use crossbeam_queue::SegQueue;
 /// use std::sync::Arc;
 ///
 /// use callbag::{for_each, from_iter, take};
@@ -81,23 +86,28 @@ use crate::{Message, Source};
 ///     }
 /// }
 ///
-/// let vec = Arc::new(ArcSwap::from_pointee(vec![]));
+/// let actual = Arc::new(SegQueue::new());
 ///
 /// let source = take(4)(from_iter(Range::new(100, 999)));
 ///
 /// for_each({
-///     let vec = Arc::clone(&vec);
+///     let actual = Arc::clone(&actual);
 ///     move |x| {
 ///         println!("{}", x);
-///         vec.rcu(move |vec| {
-///             let mut vec = (**vec).clone();
-///             vec.push(x);
-///             vec
-///         });
+///         actual.push(x);
 ///     }
 /// })(source);
 ///
-/// assert_eq!(vec.load()[..], [100, 101, 102, 103]);
+/// assert_eq!(
+///     &{
+///         let mut v = vec![];
+///         for _i in 0..actual.len() {
+///             v.push(actual.pop().unwrap());
+///         }
+///         v
+///     }[..],
+///     [100, 101, 102, 103]
+/// );
 /// ```
 pub fn take<T: 'static, S>(max: usize) -> Box<dyn Fn(S) -> Source<T>>
 where
@@ -119,29 +129,32 @@ where
                         move |message| match message {
                             Message::Handshake(_) => {
                                 panic!("sink handshake has already occurred");
-                            }
+                            },
                             Message::Data(_) => {
                                 panic!("sink must not send data");
-                            }
+                            },
                             Message::Pull => {
                                 if taken.load(AtomicOrdering::Acquire) < max {
                                     let source_talkback = source_talkback.load();
-                                    let source_talkback = source_talkback.as_ref().unwrap();
+                                    let source_talkback =
+                                        source_talkback.as_ref().expect("source talkback not set");
                                     source_talkback(Message::Pull);
                                 }
-                            }
+                            },
                             Message::Error(error) => {
                                 end.store(true, AtomicOrdering::Release);
                                 let source_talkback = source_talkback.load();
-                                let source_talkback = source_talkback.as_ref().unwrap();
+                                let source_talkback =
+                                    source_talkback.as_ref().expect("source talkback not set");
                                 source_talkback(Message::Error(error));
-                            }
+                            },
                             Message::Terminate => {
                                 end.store(true, AtomicOrdering::Release);
                                 let source_talkback = source_talkback.load();
-                                let source_talkback = source_talkback.as_ref().unwrap();
+                                let source_talkback =
+                                    source_talkback.as_ref().expect("source talkback not set");
                                 source_talkback(Message::Terminate);
-                            }
+                            },
                         }
                     }
                     .into(),
@@ -151,7 +164,7 @@ where
                         Message::Handshake(source) => {
                             source_talkback.store(Some(source));
                             sink(Message::Handshake(Arc::clone(&talkback)));
-                        }
+                        },
                         Message::Data(data) => {
                             if taken.load(AtomicOrdering::Acquire) < max {
                                 let taken = taken.fetch_add(1, AtomicOrdering::AcqRel) + 1;
@@ -160,22 +173,24 @@ where
                                     end.store(true, AtomicOrdering::Release);
                                     {
                                         let source_talkback = source_talkback.load();
-                                        let source_talkback = source_talkback.as_ref().unwrap();
+                                        let source_talkback = source_talkback
+                                            .as_ref()
+                                            .expect("source talkback not set");
                                         source_talkback(Message::Terminate);
                                     }
                                     sink(Message::Terminate);
                                 }
                             }
-                        }
+                        },
                         Message::Pull => {
                             panic!("source must not pull");
-                        }
+                        },
                         Message::Error(error) => {
                             sink(Message::Error(error));
-                        }
+                        },
                         Message::Terminate => {
                             sink(Message::Terminate);
-                        }
+                        },
                     })
                     .into(),
                 )))
