@@ -1,22 +1,22 @@
 use arc_swap::ArcSwapOption;
-use std::{
-    collections::VecDeque,
-    sync::{
-        atomic::{AtomicUsize, Ordering as AtomicOrdering},
-        Arc, RwLock,
-    },
+use crossbeam_queue::ArrayQueue;
+use never::Never;
+use std::sync::{
+    atomic::{AtomicUsize, Ordering as AtomicOrdering},
+    Arc, RwLock,
 };
+use tracing::info;
 
-use crate::common::MessagePredicate;
+use crate::common::VariantName;
 
 use callbag::{Message, Source};
 
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
 use {
+    async_executors::{Timer, TimerExt},
     async_nursery::{NurseExt, Nursery},
-    futures_timer::Delay,
-    never::Never,
-    std::{error::Error, pin::Pin, sync::atomic::AtomicBool, time::Duration},
+    std::{error::Error, sync::atomic::AtomicBool, time::Duration},
+    tracing_futures::Instrument,
 };
 
 #[cfg(all(target_arch = "wasm32", not(target_os = "wasi")))]
@@ -37,7 +37,8 @@ wasm_bindgen_test_configure!(run_in_browser);
 
 /// See <https://github.com/staltz/callbag-concat/blob/db3ce91a831309057e165f344a87aa1615b4774e/test.js#L4-L48>
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
-#[async_std::test]
+#[tracing::instrument]
+#[test_log::test(async_std::test)]
 #[cfg_attr(
     all(
         all(target_arch = "wasm32", not(target_os = "wasi")),
@@ -47,19 +48,24 @@ wasm_bindgen_test_configure!(run_in_browser);
 )]
 async fn it_concats_1_async_finite_listenable_source() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
+    let nursery = nursery.in_current_span();
 
-    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let downwards_expected_types = ["Handshake", "Data", "Data", "Data", "Terminate"];
+    let downwards_expected_types = {
+        let q = ArrayQueue::new(downwards_expected_types.len());
+        for v in downwards_expected_types {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
     let downwards_expected = [1, 2, 3];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let source_a = {
         let source_a_ref: Arc<RwLock<Option<Arc<Source<_>>>>> = Arc::new(RwLock::new(None));
@@ -68,19 +74,17 @@ async fn it_concats_1_async_finite_listenable_source() {
                 let nursery = nursery.clone();
                 let source_a_ref = Arc::clone(&source_a_ref);
                 move |message| {
-                    println!("up (a): {:?}", message);
+                    info!("up (a): {message:?}");
                     if let Message::Handshake(sink) = message {
                         let i = Arc::new(AtomicUsize::new(0));
-                        const DURATION: Duration = Duration::from_millis(100);
-                        let mut interval = Delay::new(DURATION);
                         nursery
-                            .clone()
                             .nurse({
+                                let nursery = nursery.clone();
                                 let sink = Arc::clone(&sink);
+                                const DURATION: Duration = Duration::from_millis(100);
                                 async move {
                                     loop {
-                                        Pin::new(&mut interval).await;
-                                        interval.reset(DURATION);
+                                        nursery.sleep(DURATION).await;
                                         let i = i.fetch_add(1, AtomicOrdering::AcqRel) + 1;
                                         sink(Message::Data(i));
                                         if i == 3 {
@@ -109,17 +113,19 @@ async fn it_concats_1_async_finite_listenable_source() {
     };
 
     let sink = Arc::new(
-        (move |message| {
-            println!("down: {:?}", message);
+        (move |message: Message<_, Never>| {
+            info!("down: {message:?}");
             {
-                let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
-                let et = downwards_expected_types.pop_front().unwrap();
-                assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                let et = downwards_expected_types.pop().unwrap();
+                assert_eq!(
+                    message.variant_name(),
+                    et,
+                    "downwards type is expected: {et}"
+                );
             }
             if let Message::Data(data) = message {
-                let downwards_expected = &mut *downwards_expected.write().unwrap();
-                let e = downwards_expected.pop_front().unwrap();
-                assert_eq!(data, e, "downwards data is expected: {}", e);
+                let e = downwards_expected.pop().unwrap();
+                assert_eq!(data, e, "downwards data is expected: {e}");
             }
         })
         .into(),
@@ -128,15 +134,15 @@ async fn it_concats_1_async_finite_listenable_source() {
     let source = callbag::concat!(source_a);
     source(Message::Handshake(sink));
 
+    let nursery_out = nursery.timeout(Duration::from_millis(700), nursery_out);
     drop(nursery);
-    async_std::future::timeout(Duration::from_millis(700), nursery_out)
-        .await
-        .ok();
+    nursery_out.await.ok();
 }
 
 /// See <https://github.com/staltz/callbag-concat/blob/db3ce91a831309057e165f344a87aa1615b4774e/test.js#L50-L113>
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
-#[async_std::test]
+#[tracing::instrument]
+#[test_log::test(async_std::test)]
 #[cfg_attr(
     all(
         all(target_arch = "wasm32", not(target_os = "wasi")),
@@ -146,21 +152,32 @@ async fn it_concats_1_async_finite_listenable_source() {
 )]
 async fn it_concats_2_async_finite_listenable_sources() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
+    let nursery = nursery.in_current_span();
 
-    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
+    let downwards_expected_types = [
+        "Handshake",
+        "Data",
+        "Data",
+        "Data",
+        "Data",
+        "Data",
+        "Terminate",
     ];
-    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let downwards_expected_types = {
+        let q = ArrayQueue::new(downwards_expected_types.len());
+        for v in downwards_expected_types {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
     let downwards_expected = ["1", "2", "3", "a", "b"];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let source_a = {
         let source_a_ref: Arc<RwLock<Option<Arc<Source<_>>>>> = Arc::new(RwLock::new(None));
@@ -169,19 +186,17 @@ async fn it_concats_2_async_finite_listenable_sources() {
                 let nursery = nursery.clone();
                 let source_a_ref = Arc::clone(&source_a_ref);
                 move |message| {
-                    println!("up (a): {:?}", message);
+                    info!("up (a): {message:?}");
                     if let Message::Handshake(sink) = message {
                         let i = Arc::new(AtomicUsize::new(0));
-                        const DURATION: Duration = Duration::from_millis(100);
-                        let mut interval = Delay::new(DURATION);
                         nursery
-                            .clone()
                             .nurse({
+                                let nursery = nursery.clone();
                                 let sink = Arc::clone(&sink);
+                                const DURATION: Duration = Duration::from_millis(100);
                                 async move {
                                     loop {
-                                        Pin::new(&mut interval).await;
-                                        interval.reset(DURATION);
+                                        nursery.sleep(DURATION).await;
                                         let i = i.fetch_add(1, AtomicOrdering::AcqRel) + 1;
                                         sink(Message::Data(format!("{}", i)));
                                         if i == 3 {
@@ -216,47 +231,41 @@ async fn it_concats_2_async_finite_listenable_sources() {
                 let nursery = nursery.clone();
                 let source_b_ref = Arc::clone(&source_b_ref);
                 move |message: Message<Never, String>| {
-                    println!("up (b): {:?}", message);
+                    info!("up (b): {message:?}");
                     if let Message::Handshake(sink) = message {
-                        {
-                            let timeout = Delay::new(Duration::from_millis(230));
-                            nursery
-                                .clone()
-                                .nurse({
-                                    let sink = Arc::clone(&sink);
-                                    async move {
-                                        timeout.await;
-                                        sink(Message::Data("a".to_owned()));
-                                    }
-                                })
-                                .unwrap();
-                        }
-                        {
-                            let timeout = Delay::new(Duration::from_millis(460));
-                            nursery
-                                .clone()
-                                .nurse({
-                                    let sink = Arc::clone(&sink);
-                                    async move {
-                                        timeout.await;
-                                        sink(Message::Data("b".to_owned()));
-                                    }
-                                })
-                                .unwrap();
-                        }
-                        {
-                            let timeout = Delay::new(Duration::from_millis(550));
-                            nursery
-                                .clone()
-                                .nurse({
-                                    let sink = Arc::clone(&sink);
-                                    async move {
-                                        timeout.await;
-                                        sink(Message::Terminate);
-                                    }
-                                })
-                                .unwrap();
-                        }
+                        nursery
+                            .nurse({
+                                let nursery = nursery.clone();
+                                let sink = Arc::clone(&sink);
+                                const DURATION: Duration = Duration::from_millis(230);
+                                async move {
+                                    nursery.sleep(DURATION).await;
+                                    sink(Message::Data("a".to_owned()));
+                                }
+                            })
+                            .unwrap();
+                        nursery
+                            .nurse({
+                                let nursery = nursery.clone();
+                                let sink = Arc::clone(&sink);
+                                const DURATION: Duration = Duration::from_millis(460);
+                                async move {
+                                    nursery.sleep(DURATION).await;
+                                    sink(Message::Data("b".to_owned()));
+                                }
+                            })
+                            .unwrap();
+                        nursery
+                            .nurse({
+                                let nursery = nursery.clone();
+                                let sink = Arc::clone(&sink);
+                                const DURATION: Duration = Duration::from_millis(550);
+                                async move {
+                                    nursery.sleep(DURATION).await;
+                                    sink(Message::Terminate);
+                                }
+                            })
+                            .unwrap();
                         let source_b = {
                             let source_b_ref = &mut *source_b_ref.write().unwrap();
                             source_b_ref.take().unwrap()
@@ -275,17 +284,19 @@ async fn it_concats_2_async_finite_listenable_sources() {
     };
 
     let sink = Arc::new(
-        (move |message| {
-            println!("down: {:?}", message);
+        (move |message: Message<_, Never>| {
+            info!("down: {message:?}");
             {
-                let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
-                let et = downwards_expected_types.pop_front().unwrap();
-                assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                let et = downwards_expected_types.pop().unwrap();
+                assert_eq!(
+                    message.variant_name(),
+                    et,
+                    "downwards type is expected: {et}"
+                );
             }
             if let Message::Data(data) = message {
-                let downwards_expected = &mut *downwards_expected.write().unwrap();
-                let e = downwards_expected.pop_front().unwrap();
-                assert_eq!(data, e, "downwards data is expected: {}", e);
+                let e = downwards_expected.pop().unwrap();
+                assert_eq!(data, e, "downwards data is expected: {e}");
             }
         })
         .into(),
@@ -294,49 +305,60 @@ async fn it_concats_2_async_finite_listenable_sources() {
     let source = callbag::concat!(source_a, source_b);
     source(Message::Handshake(sink));
 
+    let nursery_out = nursery.timeout(Duration::from_millis(1_200), nursery_out);
     drop(nursery);
-    async_std::future::timeout(Duration::from_millis(1200), nursery_out)
-        .await
-        .ok();
+    nursery_out.await.ok();
 }
 
 /// See <https://github.com/staltz/callbag-concat/blob/db3ce91a831309057e165f344a87aa1615b4774e/test.js#L115-L215>
-#[test]
+#[tracing::instrument]
+#[test_log::test]
 #[cfg_attr(
     all(target_arch = "wasm32", not(target_os = "wasi")),
     wasm_bindgen_test
 )]
 fn it_concats_2_sync_finite_pullable_sources() {
-    let upwards_expected_a: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-    ];
-    let upwards_expected_a: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(upwards_expected_a.into()));
-    let upwards_expected_b: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-    ];
-    let upwards_expected_b: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(upwards_expected_b.into()));
+    let upwards_expected_a = ["Pull", "Pull", "Pull", "Pull"];
+    let upwards_expected_a = {
+        let q = ArrayQueue::new(upwards_expected_a.len());
+        for v in upwards_expected_a {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
+    let upwards_expected_b = ["Pull", "Pull", "Pull"];
+    let upwards_expected_b = {
+        let q = ArrayQueue::new(upwards_expected_b.len());
+        for v in upwards_expected_b {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
-    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
+    let downwards_expected_types = [
+        "Handshake",
+        "Data",
+        "Data",
+        "Data",
+        "Data",
+        "Data",
+        "Terminate",
     ];
-    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let downwards_expected_types = {
+        let q = ArrayQueue::new(downwards_expected_types.len());
+        for v in downwards_expected_types {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
     let downwards_expected = ["10", "20", "30", "a", "b"];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let source_a = {
         let sent_a = Arc::new(AtomicUsize::new(0));
@@ -346,7 +368,7 @@ fn it_concats_2_sync_finite_pullable_sources() {
             {
                 let source_a_ref = Arc::clone(&source_a_ref);
                 move |message| {
-                    println!("up (a): {:?}", message);
+                    info!("up (a): {message:?}");
                     if let Message::Handshake(sink) = message {
                         sink_a.store(Some(sink));
                         let source_a = {
@@ -363,14 +385,10 @@ fn it_concats_2_sync_finite_pullable_sources() {
                         let sink_a = sink_a.as_ref().unwrap();
                         sink_a(Message::Terminate);
                     }
+                    assert!(!upwards_expected_a.is_empty(), "source can be pulled");
                     {
-                        let upwards_expected_a = &*upwards_expected_a.read().unwrap();
-                        assert!(!upwards_expected_a.is_empty(), "source can be pulled");
-                    }
-                    {
-                        let upwards_expected_a = &mut *upwards_expected_a.write().unwrap();
-                        let e = upwards_expected_a.pop_front().unwrap();
-                        assert!(e.0(&message), "upwards A type is expected: {}", e.1);
+                        let e = upwards_expected_a.pop().unwrap();
+                        assert_eq!(message.variant_name(), e, "upwards A type is expected: {e}");
                     }
                     if sent_a.load(AtomicOrdering::Acquire) == 0 {
                         sent_a.fetch_add(1, AtomicOrdering::AcqRel);
@@ -411,7 +429,7 @@ fn it_concats_2_sync_finite_pullable_sources() {
             {
                 let source_b_ref = Arc::clone(&source_b_ref);
                 move |message| {
-                    println!("up (b): {:?}", message);
+                    info!("up (b): {message:?}");
                     if let Message::Handshake(sink) = message {
                         sink_b.store(Some(sink));
                         let source_b = {
@@ -428,14 +446,10 @@ fn it_concats_2_sync_finite_pullable_sources() {
                         let sink_b = sink_b.as_ref().unwrap();
                         sink_b(Message::Terminate);
                     }
+                    assert!(!upwards_expected_b.is_empty(), "source can be pulled");
                     {
-                        let upwards_expected_b = &*upwards_expected_b.read().unwrap();
-                        assert!(!upwards_expected_b.is_empty(), "source can be pulled");
-                    }
-                    {
-                        let upwards_expected_b = &mut *upwards_expected_b.write().unwrap();
-                        let e = upwards_expected_b.pop_front().unwrap();
-                        assert!(e.0(&message), "upwards B type is expected: {}", e.1);
+                        let e = upwards_expected_b.pop().unwrap();
+                        assert_eq!(message.variant_name(), e, "upwards B type is expected: {e}");
                     }
                     if sent_b.load(AtomicOrdering::Acquire) == 0 {
                         sent_b.fetch_add(1, AtomicOrdering::AcqRel);
@@ -464,12 +478,15 @@ fn it_concats_2_sync_finite_pullable_sources() {
     let sink = {
         let talkback = ArcSwapOption::from(None);
         Arc::new(
-            (move |message| {
-                println!("down: {:?}", message);
+            (move |message: Message<_, Never>| {
+                info!("down: {message:?}");
                 {
-                    let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
-                    let et = downwards_expected_types.pop_front().unwrap();
-                    assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                    let et = downwards_expected_types.pop().unwrap();
+                    assert_eq!(
+                        message.variant_name(),
+                        et,
+                        "downwards type is expected: {et}"
+                    );
                 }
                 if let Message::Handshake(source) = message {
                     talkback.store(Some(source));
@@ -478,9 +495,8 @@ fn it_concats_2_sync_finite_pullable_sources() {
                     talkback(Message::Pull);
                 } else if let Message::Data(data) = message {
                     {
-                        let downwards_expected = &mut *downwards_expected.write().unwrap();
-                        let e = downwards_expected.pop_front().unwrap();
-                        assert_eq!(data, e, "downwards data is expected: {}", e);
+                        let e = downwards_expected.pop().unwrap();
+                        assert_eq!(data, e, "downwards data is expected: {e}");
                     }
                     let talkback = talkback.load();
                     let talkback = talkback.as_ref().unwrap();
@@ -497,7 +513,8 @@ fn it_concats_2_sync_finite_pullable_sources() {
 
 /// See <https://github.com/staltz/callbag-concat/blob/db3ce91a831309057e165f344a87aa1615b4774e/test.js#L217-L279>
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
-#[async_std::test]
+#[tracing::instrument]
+#[test_log::test(async_std::test)]
 #[cfg_attr(
     all(
         all(target_arch = "wasm32", not(target_os = "wasi")),
@@ -507,23 +524,32 @@ fn it_concats_2_sync_finite_pullable_sources() {
 )]
 async fn it_returns_a_source_that_disposes_upon_upwards_end() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
+    let nursery = nursery.in_current_span();
 
-    let upwards_expected: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let upwards_expected: Arc<RwLock<VecDeque<_>>> = Arc::new(RwLock::new(upwards_expected.into()));
-    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-    ];
-    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let upwards_expected = ["Handshake", "Terminate"];
+    let upwards_expected = {
+        let q = ArrayQueue::new(upwards_expected.len());
+        for v in upwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
+    let downwards_expected_types = ["Handshake", "Data", "Data", "Data"];
+    let downwards_expected_types = {
+        let q = ArrayQueue::new(downwards_expected_types.len());
+        for v in downwards_expected_types {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
     let downwards_expected = [10, 20, 30];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let make_source = {
         let nursery = nursery.clone();
@@ -534,29 +560,26 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
             let source = Arc::new(
                 {
                     let source_ref = Arc::clone(&source_ref);
-                    move |message| {
-                        println!("up: {:?}", message);
+                    move |message: Message<Never, _>| {
+                        info!("up: {message:?}");
                         let interval_cleared = Arc::clone(&interval_cleared);
                         {
-                            let upwards_expected = &mut *upwards_expected.write().unwrap();
-                            let e = upwards_expected.pop_front().unwrap();
-                            assert!(e.0(&message), "upwards type is expected: {}", e.1);
+                            let e = upwards_expected.pop().unwrap();
+                            assert_eq!(message.variant_name(), e, "upwards type is expected: {e}");
                         }
                         if let Message::Handshake(sink) = message {
-                            const DURATION: Duration = Duration::from_millis(100);
-                            let mut interval = Delay::new(DURATION);
                             nursery
-                                .clone()
                                 .nurse({
+                                    let nursery = nursery.clone();
                                     let sent = Arc::clone(&sent);
                                     let sink = Arc::clone(&sink);
+                                    const DURATION: Duration = Duration::from_millis(100);
                                     async move {
                                         loop {
-                                            Pin::new(&mut interval).await;
+                                            nursery.sleep(DURATION).await;
                                             if interval_cleared.load(AtomicOrdering::Acquire) {
                                                 break;
                                             }
-                                            interval.reset(DURATION);
                                             let sent =
                                                 sent.fetch_add(1, AtomicOrdering::AcqRel) + 1;
                                             sink(Message::Data(sent * 10));
@@ -587,21 +610,22 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
     let make_sink = move || {
         let talkback = ArcSwapOption::from(None);
         Arc::new(
-            (move |message| {
-                println!("down: {:?}", message);
+            (move |message: Message<_, Never>| {
+                info!("down: {message:?}");
                 {
-                    let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
-                    let et = downwards_expected_types.pop_front().unwrap();
-                    assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                    let et = downwards_expected_types.pop().unwrap();
+                    assert_eq!(
+                        message.variant_name(),
+                        et,
+                        "downwards type is expected: {et}"
+                    );
                 }
                 if let Message::Handshake(source) = message {
                     talkback.store(Some(source));
                 } else if let Message::Data(data) = message {
-                    let downwards_expected = &mut *downwards_expected.write().unwrap();
-                    let e = downwards_expected.pop_front().unwrap();
-                    assert_eq!(data, e, "downwards data is expected: {:?}", e);
+                    let e = downwards_expected.pop().unwrap();
+                    assert_eq!(data, e, "downwards data is expected: {e:?}");
                 }
-                let downwards_expected = &*downwards_expected.read().unwrap();
                 if downwards_expected.is_empty() {
                     let talkback = talkback.load();
                     let talkback = talkback.as_ref().unwrap();
@@ -616,15 +640,15 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
     let sink = make_sink();
     source(Message::Handshake(sink));
 
+    let nursery_out = nursery.timeout(Duration::from_millis(700), nursery_out);
     drop(nursery);
-    async_std::future::timeout(Duration::from_millis(700), nursery_out)
-        .await
-        .ok();
+    nursery_out.await.ok();
 }
 
 /// See <https://github.com/staltz/callbag-concat/blob/db3ce91a831309057e165f344a87aa1615b4774e/test.js#L333-L381>
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
-#[async_std::test]
+#[tracing::instrument]
+#[test_log::test(async_std::test)]
 #[cfg_attr(
     all(
         all(target_arch = "wasm32", not(target_os = "wasi")),
@@ -634,15 +658,16 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
 )]
 async fn it_propagates_source_error_to_sink_and_doesnt_subscribe_to_next_source() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
+    let nursery = nursery.in_current_span();
 
-    let downwards_expected: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Error(_)), "Message::Error"),
-    ];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = ["Handshake", "Data", "Data", "Error"];
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let make_source = {
         let nursery = nursery.clone();
@@ -655,20 +680,18 @@ async fn it_propagates_source_error_to_sink_and_doesnt_subscribe_to_next_source(
                     let nursery = nursery.clone();
                     let source_ref = Arc::clone(&source_ref);
                     move |message| {
-                        println!("up: {:?}", message);
+                        info!("up: {message:?}");
                         if let Message::Handshake(sink) = message {
-                            const DURATION: Duration = Duration::from_millis(100);
-                            let mut interval = Delay::new(DURATION);
                             nursery
-                                .clone()
                                 .nurse({
+                                    let nursery = nursery.clone();
                                     let limit = Arc::clone(&limit);
                                     let value = Arc::clone(&value);
                                     let sink = Arc::clone(&sink);
+                                    const DURATION: Duration = Duration::from_millis(100);
                                     async move {
                                         loop {
-                                            Pin::new(&mut interval).await;
-                                            interval.reset(DURATION);
+                                            nursery.sleep(DURATION).await;
                                             let value = value.fetch_add(1, AtomicOrdering::AcqRel);
                                             sink(Message::Data(value));
 
@@ -708,12 +731,15 @@ async fn it_propagates_source_error_to_sink_and_doesnt_subscribe_to_next_source(
 
     let make_sink = move || {
         Arc::new(
-            (move |message| {
-                println!("down: {:?}", message);
+            (move |message: Message<_, Never>| {
+                info!("down: {message:?}");
                 {
-                    let downwards_expected = &mut *downwards_expected.write().unwrap();
-                    let et = downwards_expected.pop_front().unwrap();
-                    assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                    let et = downwards_expected.pop().unwrap();
+                    assert_eq!(
+                        message.variant_name(),
+                        et,
+                        "downwards type is expected: {et}"
+                    );
                 }
             })
             .into(),
@@ -724,8 +750,7 @@ async fn it_propagates_source_error_to_sink_and_doesnt_subscribe_to_next_source(
     let sink = make_sink();
     source(Message::Handshake(sink));
 
+    let nursery_out = nursery.timeout(Duration::from_millis(700), nursery_out);
     drop(nursery);
-    async_std::future::timeout(Duration::from_millis(700), nursery_out)
-        .await
-        .ok();
+    nursery_out.await.ok();
 }
