@@ -1,15 +1,15 @@
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
-use crate::common::MessagePredicate;
+use crate::common::VariantName;
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
 use callbag::{skip, Message, Source};
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
 use {
     arc_swap::ArcSwapOption,
+    async_executors::{Timer, TimerExt},
     async_nursery::{NurseExt, Nursery},
-    futures_timer::Delay,
+    crossbeam_queue::ArrayQueue,
+    never::Never,
     std::{
-        collections::VecDeque,
-        pin::Pin,
         sync::{
             atomic::AtomicBool,
             atomic::{AtomicUsize, Ordering as AtomicOrdering},
@@ -17,6 +17,8 @@ use {
         },
         time::Duration,
     },
+    tracing::info,
+    tracing_futures::Instrument,
 };
 
 #[cfg(all(
@@ -40,7 +42,8 @@ wasm_bindgen_test_configure!(run_in_browser);
 
 /// See <https://github.com/staltz/callbag-skip/blob/698d6b7805c9bcddac038ceff25a0f0362adb25a/test.js#L4-L94>
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
-#[async_std::test]
+#[tracing::instrument]
+#[test_log::test(async_std::test)]
 #[cfg_attr(
     all(
         all(target_arch = "wasm32", not(target_os = "wasi")),
@@ -50,28 +53,32 @@ wasm_bindgen_test_configure!(run_in_browser);
 )]
 async fn it_skips_from_a_pullable_source() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
+    let nursery = nursery.in_current_span();
 
-    let upwards_expected: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-    ];
-    let upwards_expected: Arc<RwLock<VecDeque<_>>> = Arc::new(RwLock::new(upwards_expected.into()));
-    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let upwards_expected = ["Handshake", "Pull", "Pull", "Pull", "Pull", "Pull", "Pull"];
+    let upwards_expected = {
+        let q = ArrayQueue::new(upwards_expected.len());
+        for v in upwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
+    let downwards_expected_types = ["Handshake", "Data", "Data", "Terminate"];
+    let downwards_expected_types = {
+        let q = ArrayQueue::new(downwards_expected_types.len());
+        for v in downwards_expected_types {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
     let downwards_expected = [40, 50];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let make_source = {
         let nursery = nursery.clone();
@@ -82,12 +89,11 @@ async fn it_skips_from_a_pullable_source() {
             let source = Arc::new(
                 {
                     let source_ref = Arc::clone(&source_ref);
-                    move |message| {
-                        println!("up: {:?}", message);
+                    move |message: Message<Never, _>| {
+                        info!("up: {message:?}");
                         {
-                            let upwards_expected = &mut *upwards_expected.write().unwrap();
-                            let e = upwards_expected.pop_front().unwrap();
-                            assert!(e.0(&message), "upwards type is expected: {}", e.1);
+                            let e = upwards_expected.pop().unwrap();
+                            assert_eq!(message.variant_name(), e, "upwards type is expected: {e}");
                         }
 
                         if let Message::Handshake(sink) = message {
@@ -108,7 +114,6 @@ async fn it_skips_from_a_pullable_source() {
                         if sent.load(AtomicOrdering::Acquire) == 0 {
                             sent.fetch_add(1, AtomicOrdering::AcqRel);
                             nursery
-                                .clone()
                                 .nurse({
                                     let sink_ref = Arc::clone(&sink_ref);
                                     async move {
@@ -123,7 +128,6 @@ async fn it_skips_from_a_pullable_source() {
                         if sent.load(AtomicOrdering::Acquire) == 1 {
                             sent.fetch_add(1, AtomicOrdering::AcqRel);
                             nursery
-                                .clone()
                                 .nurse({
                                     let sink_ref = Arc::clone(&sink_ref);
                                     async move {
@@ -138,7 +142,6 @@ async fn it_skips_from_a_pullable_source() {
                         if sent.load(AtomicOrdering::Acquire) == 2 {
                             sent.fetch_add(1, AtomicOrdering::AcqRel);
                             nursery
-                                .clone()
                                 .nurse({
                                     let sink_ref = Arc::clone(&sink_ref);
                                     async move {
@@ -153,7 +156,6 @@ async fn it_skips_from_a_pullable_source() {
                         if sent.load(AtomicOrdering::Acquire) == 3 {
                             sent.fetch_add(1, AtomicOrdering::AcqRel);
                             nursery
-                                .clone()
                                 .nurse({
                                     let sink_ref = Arc::clone(&sink_ref);
                                     async move {
@@ -168,7 +170,6 @@ async fn it_skips_from_a_pullable_source() {
                         if sent.load(AtomicOrdering::Acquire) == 4 {
                             sent.fetch_add(1, AtomicOrdering::AcqRel);
                             nursery
-                                .clone()
                                 .nurse({
                                     let sink_ref = Arc::clone(&sink_ref);
                                     async move {
@@ -183,7 +184,6 @@ async fn it_skips_from_a_pullable_source() {
                         if sent.load(AtomicOrdering::Acquire) == 5 {
                             sent.fetch_add(1, AtomicOrdering::AcqRel);
                             nursery
-                                .clone()
                                 .nurse({
                                     let sink_ref = Arc::clone(&sink_ref);
                                     async move {
@@ -209,12 +209,15 @@ async fn it_skips_from_a_pullable_source() {
     let make_sink = move || {
         let talkback = ArcSwapOption::from(None);
         Arc::new(
-            (move |message| {
-                println!("down: {:?}", message);
+            (move |message: Message<_, Never>| {
+                info!("down: {message:?}");
                 {
-                    let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
-                    let et = downwards_expected_types.pop_front().unwrap();
-                    assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                    let et = downwards_expected_types.pop().unwrap();
+                    assert_eq!(
+                        message.variant_name(),
+                        et,
+                        "downwards type is expected: {et}"
+                    );
                 }
                 if let Message::Handshake(source) = message {
                     talkback.store(Some(source));
@@ -223,9 +226,8 @@ async fn it_skips_from_a_pullable_source() {
                     talkback(Message::Pull);
                 } else if let Message::Data(data) = message {
                     {
-                        let downwards_expected = &mut *downwards_expected.write().unwrap();
-                        let e = downwards_expected.pop_front().unwrap();
-                        assert_eq!(data, e, "downwards data is expected: {}", e);
+                        let e = downwards_expected.pop().unwrap();
+                        assert_eq!(data, e, "downwards data is expected: {e}");
                     }
                     let talkback = talkback.load();
                     let talkback = talkback.as_ref().unwrap();
@@ -241,15 +243,15 @@ async fn it_skips_from_a_pullable_source() {
     let sink = make_sink();
     skipped(Message::Handshake(sink));
 
+    let nursery_out = nursery.timeout(Duration::from_millis(300), nursery_out);
     drop(nursery);
-    async_std::future::timeout(Duration::from_millis(300), nursery_out)
-        .await
-        .ok();
+    nursery_out.await.ok();
 }
 
 /// See <https://github.com/staltz/callbag-skip/blob/698d6b7805c9bcddac038ceff25a0f0362adb25a/test.js#L96-L152>
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
-#[async_std::test]
+#[tracing::instrument]
+#[test_log::test(async_std::test)]
 #[cfg_attr(
     all(
         all(target_arch = "wasm32", not(target_os = "wasi")),
@@ -259,25 +261,32 @@ async fn it_skips_from_a_pullable_source() {
 )]
 async fn it_skips_an_async_listenable_source() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
+    let nursery = nursery.in_current_span();
 
-    let upwards_expected: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-    ];
-    let upwards_expected: Arc<RwLock<VecDeque<_>>> = Arc::new(RwLock::new(upwards_expected.into()));
-    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let upwards_expected = ["Handshake", "Pull", "Pull", "Pull"];
+    let upwards_expected = {
+        let q = ArrayQueue::new(upwards_expected.len());
+        for v in upwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
+    let downwards_expected_types = ["Handshake", "Data", "Data", "Terminate"];
+    let downwards_expected_types = {
+        let q = ArrayQueue::new(downwards_expected_types.len());
+        for v in downwards_expected_types {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
     let downwards_expected = [40, 50];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let make_source = {
         let nursery = nursery.clone();
@@ -286,80 +295,69 @@ async fn it_skips_an_async_listenable_source() {
             let source = Arc::new(
                 {
                     let source_ref = Arc::clone(&source_ref);
-                    move |message| {
-                        println!("up: {:?}", message);
+                    move |message: Message<Never, _>| {
+                        info!("up: {message:?}");
                         {
-                            let upwards_expected = &mut *upwards_expected.write().unwrap();
-                            let e = upwards_expected.pop_front().unwrap();
-                            assert!(e.0(&message), "upwards type is expected: {}", e.1);
+                            let e = upwards_expected.pop().unwrap();
+                            assert_eq!(message.variant_name(), e, "upwards type is expected: {e}");
                         }
                         if let Message::Handshake(sink) = message {
-                            {
-                                let timeout = Delay::new(Duration::from_millis(100));
-                                nursery
-                                    .clone()
-                                    .nurse({
-                                        let sink = Arc::clone(&sink);
-                                        async move {
-                                            timeout.await;
-                                            sink(Message::Data(10));
-                                        }
-                                    })
-                                    .unwrap();
-                            }
-                            {
-                                let timeout = Delay::new(Duration::from_millis(200));
-                                nursery
-                                    .clone()
-                                    .nurse({
-                                        let sink = Arc::clone(&sink);
-                                        async move {
-                                            timeout.await;
-                                            sink(Message::Data(20));
-                                        }
-                                    })
-                                    .unwrap();
-                            }
-                            {
-                                let timeout = Delay::new(Duration::from_millis(300));
-                                nursery
-                                    .clone()
-                                    .nurse({
-                                        let sink = Arc::clone(&sink);
-                                        async move {
-                                            timeout.await;
-                                            sink(Message::Data(30));
-                                        }
-                                    })
-                                    .unwrap();
-                            }
-                            {
-                                let timeout = Delay::new(Duration::from_millis(400));
-                                nursery
-                                    .clone()
-                                    .nurse({
-                                        let sink = Arc::clone(&sink);
-                                        async move {
-                                            timeout.await;
-                                            sink(Message::Data(40));
-                                        }
-                                    })
-                                    .unwrap();
-                            }
-                            {
-                                let timeout = Delay::new(Duration::from_millis(500));
-                                nursery
-                                    .clone()
-                                    .nurse({
-                                        let sink = Arc::clone(&sink);
-                                        async move {
-                                            timeout.await;
-                                            sink(Message::Data(50));
-                                            sink(Message::Terminate);
-                                        }
-                                    })
-                                    .unwrap();
-                            }
+                            nursery
+                                .nurse({
+                                    let nursery = nursery.clone();
+                                    let sink = Arc::clone(&sink);
+                                    const DURATION: Duration = Duration::from_millis(100);
+                                    async move {
+                                        nursery.sleep(DURATION).await;
+                                        sink(Message::Data(10));
+                                    }
+                                })
+                                .unwrap();
+                            nursery
+                                .nurse({
+                                    let nursery = nursery.clone();
+                                    let sink = Arc::clone(&sink);
+                                    const DURATION: Duration = Duration::from_millis(200);
+                                    async move {
+                                        nursery.sleep(DURATION).await;
+                                        sink(Message::Data(20));
+                                    }
+                                })
+                                .unwrap();
+                            nursery
+                                .nurse({
+                                    let nursery = nursery.clone();
+                                    let sink = Arc::clone(&sink);
+                                    const DURATION: Duration = Duration::from_millis(300);
+                                    async move {
+                                        nursery.sleep(DURATION).await;
+                                        sink(Message::Data(30));
+                                    }
+                                })
+                                .unwrap();
+                            nursery
+                                .nurse({
+                                    let nursery = nursery.clone();
+                                    let sink = Arc::clone(&sink);
+                                    const DURATION: Duration = Duration::from_millis(400);
+                                    async move {
+                                        nursery.sleep(DURATION).await;
+                                        sink(Message::Data(40));
+                                    }
+                                })
+                                .unwrap();
+                            nursery
+                                .nurse({
+                                    let nursery = nursery.clone();
+                                    let sink = Arc::clone(&sink);
+                                    const DURATION: Duration = Duration::from_millis(500);
+                                    async move {
+                                        nursery.sleep(DURATION).await;
+                                        sink(Message::Data(50));
+                                        sink(Message::Terminate);
+                                    }
+                                })
+                                .unwrap();
                             let source = {
                                 let source_ref = &mut *source_ref.write().unwrap();
                                 source_ref.take().unwrap()
@@ -379,17 +377,19 @@ async fn it_skips_an_async_listenable_source() {
     };
 
     let sink = Arc::new(
-        (move |message| {
-            println!("down: {:?}", message);
+        (move |message: Message<_, Never>| {
+            info!("down: {message:?}");
             {
-                let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
-                let et = downwards_expected_types.pop_front().unwrap();
-                assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                let et = downwards_expected_types.pop().unwrap();
+                assert_eq!(
+                    message.variant_name(),
+                    et,
+                    "downwards type is expected: {et}"
+                );
             }
             if let Message::Data(data) = message {
-                let downwards_expected = &mut *downwards_expected.write().unwrap();
-                let e = downwards_expected.pop_front().unwrap();
-                assert_eq!(data, e, "downwards data is expected: {}", e);
+                let e = downwards_expected.pop().unwrap();
+                assert_eq!(data, e, "downwards data is expected: {e}");
             }
         })
         .into(),
@@ -399,15 +399,15 @@ async fn it_skips_an_async_listenable_source() {
     let skipped = skip(3)(source);
     skipped(Message::Handshake(sink));
 
+    let nursery_out = nursery.timeout(Duration::from_millis(900), nursery_out);
     drop(nursery);
-    async_std::future::timeout(Duration::from_millis(900), nursery_out)
-        .await
-        .ok();
+    nursery_out.await.ok();
 }
 
 /// See <https://github.com/staltz/callbag-skip/blob/698d6b7805c9bcddac038ceff25a0f0362adb25a/test.js#L154-L218>
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
-#[async_std::test]
+#[tracing::instrument]
+#[test_log::test(async_std::test)]
 #[cfg_attr(
     all(
         all(target_arch = "wasm32", not(target_os = "wasi")),
@@ -417,25 +417,32 @@ async fn it_skips_an_async_listenable_source() {
 )]
 async fn it_returns_a_source_that_disposes_upon_upwards_end() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
+    let nursery = nursery.in_current_span();
 
-    let upwards_expected: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let upwards_expected: Arc<RwLock<VecDeque<_>>> = Arc::new(RwLock::new(upwards_expected.into()));
-    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-    ];
-    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let upwards_expected = ["Handshake", "Pull", "Pull", "Pull", "Terminate"];
+    let upwards_expected = {
+        let q = ArrayQueue::new(upwards_expected.len());
+        for v in upwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
+    let downwards_expected_types = ["Handshake", "Data", "Data"];
+    let downwards_expected_types = {
+        let q = ArrayQueue::new(downwards_expected_types.len());
+        for v in downwards_expected_types {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
     let downwards_expected = [40, 50];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let make_source = {
         let nursery = nursery.clone();
@@ -446,29 +453,26 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
             let source = Arc::new(
                 {
                     let source_ref = Arc::clone(&source_ref);
-                    move |message| {
-                        println!("up: {:?}", message);
+                    move |message: Message<Never, _>| {
+                        info!("up: {message:?}");
                         let interval_cleared = Arc::clone(&interval_cleared);
                         {
-                            let upwards_expected = &mut *upwards_expected.write().unwrap();
-                            let e = upwards_expected.pop_front().unwrap();
-                            assert!(e.0(&message), "upwards type is expected: {}", e.1);
+                            let e = upwards_expected.pop().unwrap();
+                            assert_eq!(message.variant_name(), e, "upwards type is expected: {e}");
                         }
                         if let Message::Handshake(sink) = message {
-                            const DURATION: Duration = Duration::from_millis(100);
-                            let mut interval = Delay::new(DURATION);
                             nursery
-                                .clone()
                                 .nurse({
+                                    let nursery = nursery.clone();
                                     let sent = Arc::clone(&sent);
                                     let sink = Arc::clone(&sink);
+                                    const DURATION: Duration = Duration::from_millis(100);
                                     async move {
                                         loop {
-                                            Pin::new(&mut interval).await;
+                                            nursery.sleep(DURATION).await;
                                             if interval_cleared.load(AtomicOrdering::Acquire) {
                                                 break;
                                             }
-                                            interval.reset(DURATION);
                                             let sent =
                                                 sent.fetch_add(1, AtomicOrdering::AcqRel) + 1;
                                             sink(Message::Data(sent * 10));
@@ -499,21 +503,22 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
     let make_sink = move || {
         let talkback = ArcSwapOption::from(None);
         Arc::new(
-            (move |message| {
-                println!("down: {:?}", message);
+            (move |message: Message<_, Never>| {
+                info!("down: {message:?}");
                 {
-                    let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
-                    let et = downwards_expected_types.pop_front().unwrap();
-                    assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                    let et = downwards_expected_types.pop().unwrap();
+                    assert_eq!(
+                        message.variant_name(),
+                        et,
+                        "downwards type is expected: {et}"
+                    );
                 }
                 if let Message::Handshake(source) = message {
                     talkback.store(Some(source));
                 } else if let Message::Data(data) = message {
-                    let downwards_expected = &mut *downwards_expected.write().unwrap();
-                    let e = downwards_expected.pop_front().unwrap();
-                    assert_eq!(data, e, "downwards data is expected: {}", e);
+                    let e = downwards_expected.pop().unwrap();
+                    assert_eq!(data, e, "downwards data is expected: {e}");
                 }
-                let downwards_expected = &*downwards_expected.read().unwrap();
                 if downwards_expected.is_empty() {
                     let talkback = talkback.load();
                     let talkback = talkback.as_ref().unwrap();
@@ -529,8 +534,7 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
     let sink = make_sink();
     skipped(Message::Handshake(sink));
 
+    let nursery_out = nursery.timeout(Duration::from_millis(800), nursery_out);
     drop(nursery);
-    async_std::future::timeout(Duration::from_millis(800), nursery_out)
-        .await
-        .ok();
+    nursery_out.await.ok();
 }
