@@ -1,21 +1,25 @@
 use arc_swap::ArcSwapOption;
-use std::{
-    collections::VecDeque,
-    sync::{
-        atomic::{AtomicUsize, Ordering as AtomicOrdering},
-        Arc, RwLock,
-    },
+use crossbeam_queue::ArrayQueue;
+use never::Never;
+use std::sync::{
+    atomic::{AtomicUsize, Ordering as AtomicOrdering},
+    Arc,
 };
+use tracing::info;
 
-use crate::common::MessagePredicate;
+use crate::common::VariantName;
 
 use callbag::{take, Message, Source};
 
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
 use {
+    async_executors::{Timer, TimerExt},
     async_nursery::{NurseExt, Nursery},
-    futures_timer::Delay,
-    std::{pin::Pin, sync::atomic::AtomicBool, time::Duration},
+    std::{
+        sync::{atomic::AtomicBool, RwLock},
+        time::Duration,
+    },
+    tracing_futures::Instrument,
 };
 
 #[cfg(all(target_arch = "wasm32", not(target_os = "wasi")))]
@@ -36,7 +40,8 @@ wasm_bindgen_test_configure!(run_in_browser);
 
 /// See <https://github.com/staltz/callbag-take/blob/6ae7755ea5f014306704450a40eb72ffdb21d308/test.js#L4-L103>
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
-#[async_std::test]
+#[tracing::instrument]
+#[test_log::test(async_std::test)]
 #[cfg_attr(
     all(
         all(target_arch = "wasm32", not(target_os = "wasi")),
@@ -46,27 +51,32 @@ wasm_bindgen_test_configure!(run_in_browser);
 )]
 async fn it_takes_from_a_pullable_source() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
+    let nursery = nursery.in_current_span();
 
-    let upwards_expected: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let upwards_expected: Arc<RwLock<VecDeque<_>>> = Arc::new(RwLock::new(upwards_expected.into()));
-    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let upwards_expected = ["Handshake", "Pull", "Pull", "Pull", "Terminate"];
+    let upwards_expected = {
+        let q = ArrayQueue::new(upwards_expected.len());
+        for v in upwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
+    let downwards_expected_types = ["Handshake", "Data", "Data", "Data", "Terminate"];
+    let downwards_expected_types = {
+        let q = ArrayQueue::new(downwards_expected_types.len());
+        for v in downwards_expected_types {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
     let downwards_expected = [10, 20, 30];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let make_source = {
         let nursery = nursery.clone();
@@ -77,12 +87,11 @@ async fn it_takes_from_a_pullable_source() {
             let source = Arc::new(
                 {
                     let source_ref = Arc::clone(&source_ref);
-                    move |message| {
-                        println!("up: {:?}", message);
+                    move |message: Message<Never, _>| {
+                        info!("up: {message:?}");
                         {
-                            let upwards_expected = &mut *upwards_expected.write().unwrap();
-                            let e = upwards_expected.pop_front().unwrap();
-                            assert!(e.0(&message), "upwards type is expected: {}", e.1);
+                            let e = upwards_expected.pop().unwrap();
+                            assert_eq!(message.variant_name(), e, "upwards type is expected: {e}");
                         }
 
                         if let Message::Handshake(sink) = message {
@@ -103,7 +112,6 @@ async fn it_takes_from_a_pullable_source() {
                         if sent.load(AtomicOrdering::Acquire) == 0 {
                             sent.fetch_add(1, AtomicOrdering::AcqRel);
                             nursery
-                                .clone()
                                 .nurse({
                                     let sink_ref = Arc::clone(&sink_ref);
                                     async move {
@@ -118,7 +126,6 @@ async fn it_takes_from_a_pullable_source() {
                         if sent.load(AtomicOrdering::Acquire) == 1 {
                             sent.fetch_add(1, AtomicOrdering::AcqRel);
                             nursery
-                                .clone()
                                 .nurse({
                                     let sink_ref = Arc::clone(&sink_ref);
                                     async move {
@@ -133,7 +140,6 @@ async fn it_takes_from_a_pullable_source() {
                         if sent.load(AtomicOrdering::Acquire) == 2 {
                             sent.fetch_add(1, AtomicOrdering::AcqRel);
                             nursery
-                                .clone()
                                 .nurse({
                                     let sink_ref = Arc::clone(&sink_ref);
                                     async move {
@@ -148,7 +154,6 @@ async fn it_takes_from_a_pullable_source() {
                         if sent.load(AtomicOrdering::Acquire) == 3 {
                             sent.fetch_add(1, AtomicOrdering::AcqRel);
                             nursery
-                                .clone()
                                 .nurse({
                                     let sink_ref = Arc::clone(&sink_ref);
                                     async move {
@@ -163,7 +168,6 @@ async fn it_takes_from_a_pullable_source() {
                         if sent.load(AtomicOrdering::Acquire) == 4 {
                             sent.fetch_add(1, AtomicOrdering::AcqRel);
                             nursery
-                                .clone()
                                 .nurse({
                                     let sink_ref = Arc::clone(&sink_ref);
                                     async move {
@@ -196,12 +200,15 @@ async fn it_takes_from_a_pullable_source() {
     let make_sink = move || {
         let talkback = ArcSwapOption::from(None);
         Arc::new(
-            (move |message| {
-                println!("down: {:?}", message);
+            (move |message: Message<_, Never>| {
+                info!("down: {message:?}");
                 {
-                    let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
-                    let et = downwards_expected_types.pop_front().unwrap();
-                    assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                    let et = downwards_expected_types.pop().unwrap();
+                    assert_eq!(
+                        message.variant_name(),
+                        et,
+                        "downwards type is expected: {et}"
+                    );
                 }
                 if let Message::Handshake(source) = message {
                     talkback.store(Some(source));
@@ -210,9 +217,8 @@ async fn it_takes_from_a_pullable_source() {
                     talkback(Message::Pull);
                 } else if let Message::Data(data) = message {
                     {
-                        let downwards_expected = &mut *downwards_expected.write().unwrap();
-                        let e = downwards_expected.pop_front().unwrap();
-                        assert_eq!(data, e, "downwards data is expected: {}", e);
+                        let e = downwards_expected.pop().unwrap();
+                        assert_eq!(data, e, "downwards data is expected: {e}");
                     }
                     let talkback = talkback.load();
                     let talkback = talkback.as_ref().unwrap();
@@ -228,15 +234,15 @@ async fn it_takes_from_a_pullable_source() {
     let sink = make_sink();
     taken(Message::Handshake(sink));
 
+    let nursery_out = nursery.timeout(Duration::from_millis(300), nursery_out);
     drop(nursery);
-    async_std::future::timeout(Duration::from_millis(300), nursery_out)
-        .await
-        .ok();
+    nursery_out.await.ok();
 }
 
 /// See <https://github.com/staltz/callbag-take/blob/6ae7755ea5f014306704450a40eb72ffdb21d308/test.js#L105-L155>
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
-#[async_std::test]
+#[tracing::instrument]
+#[test_log::test(async_std::test)]
 #[cfg_attr(
     all(
         all(target_arch = "wasm32", not(target_os = "wasi")),
@@ -246,24 +252,32 @@ async fn it_takes_from_a_pullable_source() {
 )]
 async fn it_takes_an_async_listenable_source() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
+    let nursery = nursery.in_current_span();
 
-    let upwards_expected: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let upwards_expected: Arc<RwLock<VecDeque<_>>> = Arc::new(RwLock::new(upwards_expected.into()));
-    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let upwards_expected = ["Handshake", "Terminate"];
+    let upwards_expected = {
+        let q = ArrayQueue::new(upwards_expected.len());
+        for v in upwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
+    let downwards_expected_types = ["Handshake", "Data", "Data", "Data", "Terminate"];
+    let downwards_expected_types = {
+        let q = ArrayQueue::new(downwards_expected_types.len());
+        for v in downwards_expected_types {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
     let downwards_expected = [10, 20, 30];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let make_source = {
         let nursery = nursery.clone();
@@ -274,29 +288,26 @@ async fn it_takes_an_async_listenable_source() {
             let source = Arc::new(
                 {
                     let source_ref = Arc::clone(&source_ref);
-                    move |message| {
-                        println!("up: {:?}", message);
+                    move |message: Message<Never, _>| {
+                        info!("up: {message:?}");
                         let interval_cleared = Arc::clone(&interval_cleared);
                         {
-                            let upwards_expected = &mut *upwards_expected.write().unwrap();
-                            let e = upwards_expected.pop_front().unwrap();
-                            assert!(e.0(&message), "upwards type is expected: {}", e.1);
+                            let e = upwards_expected.pop().unwrap();
+                            assert_eq!(message.variant_name(), e, "upwards type is expected: {e}");
                         }
                         if let Message::Handshake(sink) = message {
-                            const DURATION: Duration = Duration::from_millis(100);
-                            let mut interval = Delay::new(DURATION);
                             nursery
-                                .clone()
                                 .nurse({
+                                    let nursery = nursery.clone();
                                     let sent = Arc::clone(&sent);
                                     let sink = Arc::clone(&sink);
+                                    const DURATION: Duration = Duration::from_millis(100);
                                     async move {
                                         loop {
-                                            Pin::new(&mut interval).await;
+                                            nursery.sleep(DURATION).await;
                                             if interval_cleared.load(AtomicOrdering::Acquire) {
                                                 break;
                                             }
-                                            interval.reset(DURATION);
                                             let sent =
                                                 sent.fetch_add(1, AtomicOrdering::AcqRel) + 1;
                                             sink(Message::Data(sent * 10));
@@ -325,17 +336,19 @@ async fn it_takes_an_async_listenable_source() {
     };
 
     let sink = Arc::new(
-        (move |message| {
-            println!("down: {:?}", message);
+        (move |message: Message<_, Never>| {
+            info!("down: {message:?}");
             {
-                let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
-                let et = downwards_expected_types.pop_front().unwrap();
-                assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                let et = downwards_expected_types.pop().unwrap();
+                assert_eq!(
+                    message.variant_name(),
+                    et,
+                    "downwards type is expected: {et}"
+                );
             }
             if let Message::Data(data) = message {
-                let downwards_expected = &mut *downwards_expected.write().unwrap();
-                let e = downwards_expected.pop_front().unwrap();
-                assert_eq!(data, e, "downwards data is expected: {}", e);
+                let e = downwards_expected.pop().unwrap();
+                assert_eq!(data, e, "downwards data is expected: {e}");
             }
         })
         .into(),
@@ -345,15 +358,15 @@ async fn it_takes_an_async_listenable_source() {
     let taken = take(3)(source);
     taken(Message::Handshake(sink));
 
+    let nursery_out = nursery.timeout(Duration::from_millis(700), nursery_out);
     drop(nursery);
-    async_std::future::timeout(Duration::from_millis(700), nursery_out)
-        .await
-        .ok();
+    nursery_out.await.ok();
 }
 
 /// See <https://github.com/staltz/callbag-take/blob/6ae7755ea5f014306704450a40eb72ffdb21d308/test.js#L157-L216>
 #[cfg(not(all(target_arch = "wasm32", target_os = "wasi")))]
-#[async_std::test]
+#[tracing::instrument]
+#[test_log::test(async_std::test)]
 #[cfg_attr(
     all(
         all(target_arch = "wasm32", not(target_os = "wasi")),
@@ -363,23 +376,32 @@ async fn it_takes_an_async_listenable_source() {
 )]
 async fn it_returns_a_source_that_disposes_upon_upwards_end() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
+    let nursery = nursery.in_current_span();
 
-    let upwards_expected: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let upwards_expected: Arc<RwLock<VecDeque<_>>> = Arc::new(RwLock::new(upwards_expected.into()));
-    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-    ];
-    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let upwards_expected = ["Handshake", "Terminate"];
+    let upwards_expected = {
+        let q = ArrayQueue::new(upwards_expected.len());
+        for v in upwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
+    let downwards_expected_types = ["Handshake", "Data", "Data", "Data"];
+    let downwards_expected_types = {
+        let q = ArrayQueue::new(downwards_expected_types.len());
+        for v in downwards_expected_types {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
     let downwards_expected = [10, 20, 30];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let make_source = {
         let nursery = nursery.clone();
@@ -390,29 +412,26 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
             let source = Arc::new(
                 {
                     let source_ref = Arc::clone(&source_ref);
-                    move |message| {
-                        println!("up: {:?}", message);
+                    move |message: Message<Never, _>| {
+                        info!("up: {message:?}");
                         let interval_cleared = Arc::clone(&interval_cleared);
                         {
-                            let upwards_expected = &mut *upwards_expected.write().unwrap();
-                            let e = upwards_expected.pop_front().unwrap();
-                            assert!(e.0(&message), "upwards type is expected: {}", e.1);
+                            let e = upwards_expected.pop().unwrap();
+                            assert_eq!(message.variant_name(), e, "upwards type is expected: {e}");
                         }
                         if let Message::Handshake(sink) = message {
-                            const DURATION: Duration = Duration::from_millis(100);
-                            let mut interval = Delay::new(DURATION);
                             nursery
-                                .clone()
                                 .nurse({
+                                    let nursery = nursery.clone();
                                     let sent = Arc::clone(&sent);
                                     let sink = Arc::clone(&sink);
+                                    const DURATION: Duration = Duration::from_millis(100);
                                     async move {
                                         loop {
-                                            Pin::new(&mut interval).await;
+                                            nursery.sleep(DURATION).await;
                                             if interval_cleared.load(AtomicOrdering::Acquire) {
                                                 break;
                                             }
-                                            interval.reset(DURATION);
                                             let sent =
                                                 sent.fetch_add(1, AtomicOrdering::AcqRel) + 1;
                                             sink(Message::Data(sent * 10));
@@ -443,21 +462,22 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
     let make_sink = move || {
         let talkback = ArcSwapOption::from(None);
         Arc::new(
-            (move |message| {
-                println!("down: {:?}", message);
+            (move |message: Message<_, Never>| {
+                info!("down: {message:?}");
                 {
-                    let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
-                    let et = downwards_expected_types.pop_front().unwrap();
-                    assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                    let et = downwards_expected_types.pop().unwrap();
+                    assert_eq!(
+                        message.variant_name(),
+                        et,
+                        "downwards type is expected: {et}"
+                    );
                 }
                 if let Message::Handshake(source) = message {
                     talkback.store(Some(source));
                 } else if let Message::Data(data) = message {
-                    let downwards_expected = &mut *downwards_expected.write().unwrap();
-                    let e = downwards_expected.pop_front().unwrap();
-                    assert_eq!(data, e, "downwards data is expected: {}", e);
+                    let e = downwards_expected.pop().unwrap();
+                    assert_eq!(data, e, "downwards data is expected: {e}");
                 }
-                let downwards_expected = &*downwards_expected.read().unwrap();
                 if downwards_expected.is_empty() {
                     let talkback = talkback.load();
                     let talkback = talkback.as_ref().unwrap();
@@ -473,14 +493,14 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
     let sink = make_sink();
     taken(Message::Handshake(sink));
 
+    let nursery_out = nursery.timeout(Duration::from_millis(700), nursery_out);
     drop(nursery);
-    async_std::future::timeout(Duration::from_millis(700), nursery_out)
-        .await
-        .ok();
+    nursery_out.await.ok();
 }
 
 /// See <https://github.com/staltz/callbag-take/blob/6ae7755ea5f014306704450a40eb72ffdb21d308/test.js#L218-L283>
-#[test]
+#[tracing::instrument]
+#[test_log::test]
 #[cfg_attr(
     all(target_arch = "wasm32", not(target_os = "wasi")),
     wasm_bindgen_test
@@ -488,32 +508,37 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
 fn it_does_not_redundantly_terminate_a_synchronous_pullable_source() {
     let terminations = Arc::new(AtomicUsize::new(0));
 
-    let upwards_expected: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Pull), "Message::Pull"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let upwards_expected: Arc<RwLock<VecDeque<_>>> = Arc::new(RwLock::new(upwards_expected.into()));
-    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
-        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
-    ];
-    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let upwards_expected = ["Pull", "Pull", "Pull", "Terminate"];
+    let upwards_expected = {
+        let q = ArrayQueue::new(upwards_expected.len());
+        for v in upwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
+    let downwards_expected_types = ["Handshake", "Data", "Data", "Data", "Terminate"];
+    let downwards_expected_types = {
+        let q = ArrayQueue::new(downwards_expected_types.len());
+        for v in downwards_expected_types {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
     let downwards_expected = [7, 8, 9];
-    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
-        Arc::new(RwLock::new(downwards_expected.into()));
+    let downwards_expected = {
+        let q = ArrayQueue::new(downwards_expected.len());
+        for v in downwards_expected {
+            q.push(v).ok();
+        }
+        Arc::new(q)
+    };
 
     let range_infinite = {
         let terminations = Arc::clone(&terminations);
         move |start| {
             Arc::new(
                 (move |message| {
-                    println!("up: {:?}", message);
+                    info!("up: {message:?}");
                     let terminations = Arc::clone(&terminations);
                     let upwards_expected = Arc::clone(&upwards_expected);
                     if let Message::Handshake(sink) = message {
@@ -521,13 +546,15 @@ fn it_does_not_redundantly_terminate_a_synchronous_pullable_source() {
                         sink(Message::Handshake(Arc::new(
                             {
                                 let sink = Arc::clone(&sink);
-                                move |message| {
-                                    println!("up: {:?}", message);
+                                move |message: Message<Never, _>| {
+                                    info!("up: {message:?}");
                                     {
-                                        let upwards_expected =
-                                            &mut *upwards_expected.write().unwrap();
-                                        let e = upwards_expected.pop_front().unwrap();
-                                        assert!(e.0(&message), "upwards type is expected: {}", e.1);
+                                        let e = upwards_expected.pop().unwrap();
+                                        assert_eq!(
+                                            message.variant_name(),
+                                            e,
+                                            "upwards type is expected: {e}"
+                                        );
                                     }
                                     if let Message::Pull = message {
                                         let counter = counter.fetch_add(1, AtomicOrdering::AcqRel);
@@ -549,12 +576,15 @@ fn it_does_not_redundantly_terminate_a_synchronous_pullable_source() {
     let make_sink = move || {
         let talkback = ArcSwapOption::from(None);
         Arc::new(
-            (move |message| {
-                println!("down: {:?}", message);
+            (move |message: Message<_, Never>| {
+                info!("down: {message:?}");
                 {
-                    let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
-                    let et = downwards_expected_types.pop_front().unwrap();
-                    assert!(et.0(&message), "downwards type is expected: {}", et.1);
+                    let et = downwards_expected_types.pop().unwrap();
+                    assert_eq!(
+                        message.variant_name(),
+                        et,
+                        "downwards type is expected: {et}"
+                    );
                 }
                 if let Message::Handshake(source) = message {
                     talkback.store(Some(source));
@@ -563,9 +593,8 @@ fn it_does_not_redundantly_terminate_a_synchronous_pullable_source() {
                     talkback(Message::Pull);
                 } else if let Message::Data(data) = message {
                     {
-                        let downwards_expected = &mut *downwards_expected.write().unwrap();
-                        let e = downwards_expected.pop_front().unwrap();
-                        assert_eq!(data, e, "downwards data is expected: {}", e);
+                        let e = downwards_expected.pop().unwrap();
+                        assert_eq!(data, e, "downwards data is expected: {e}");
                     }
                     let talkback = talkback.load();
                     let talkback = talkback.as_ref().unwrap();
@@ -589,7 +618,8 @@ fn it_does_not_redundantly_terminate_a_synchronous_pullable_source() {
 }
 
 /// See <https://github.com/staltz/callbag-take/blob/6ae7755ea5f014306704450a40eb72ffdb21d308/test.js#L285-L351>
-#[test]
+#[tracing::instrument]
+#[test_log::test]
 #[cfg_attr(
     all(target_arch = "wasm32", not(target_os = "wasi")),
     wasm_bindgen_test
@@ -603,7 +633,7 @@ fn it_does_not_mutually_terminate_a_sink() {
         move |start| {
             Arc::new(
                 (move |message| {
-                    println!("up: {:?}", message);
+                    info!("up: {message:?}");
                     let terminations = Arc::clone(&terminations);
                     if let Message::Handshake(sink) = message {
                         let counter = Arc::new(AtomicUsize::new(start));
@@ -611,7 +641,7 @@ fn it_does_not_mutually_terminate_a_sink() {
                             {
                                 let sink = Arc::clone(&sink);
                                 move |message| {
-                                    println!("up: {:?}", message);
+                                    info!("up: {message:?}");
                                     if let Message::Pull = message {
                                         let counter = counter.fetch_add(1, AtomicOrdering::AcqRel);
                                         sink(Message::Data(counter));
@@ -633,7 +663,7 @@ fn it_does_not_mutually_terminate_a_sink() {
         let talkback = ArcSwapOption::from(None);
         Arc::new(
             (move |message| {
-                println!("down: {:?}", message);
+                info!("down: {message:?}");
                 if let Message::Handshake(source) = message {
                     talkback.store(Some(source));
                     let talkback = talkback.load();
@@ -654,7 +684,7 @@ fn it_does_not_mutually_terminate_a_sink() {
         move |source: Source<usize>| {
             Arc::new(
                 (move |message| {
-                    println!("logger: {:?}", message);
+                    info!("logger: {message:?}");
                     let logger_terminations = Arc::clone(&logger_terminations);
                     if let Message::Handshake(sink) = message {
                         let source_talkback: Arc<ArcSwapOption<Source<usize>>> =
@@ -664,7 +694,7 @@ fn it_does_not_mutually_terminate_a_sink() {
                                 let logger_terminations = Arc::clone(&logger_terminations);
                                 let source_talkback = Arc::clone(&source_talkback);
                                 move |message| {
-                                    println!("logger (from down): {:?}", message);
+                                    info!("logger (from down): {message:?}");
                                     if let Message::Error(_) | Message::Terminate = message {
                                         logger_terminations.fetch_add(1, AtomicOrdering::AcqRel);
                                     }
@@ -677,7 +707,7 @@ fn it_does_not_mutually_terminate_a_sink() {
                         );
                         source(Message::Handshake(Arc::new(
                             (move |message| {
-                                println!("logger (from up): {:?}", message);
+                                info!("logger (from up): {message:?}");
                                 if let Message::Error(_) | Message::Terminate = message {
                                     logger_terminations.fetch_add(1, AtomicOrdering::AcqRel);
                                 } else if let Message::Handshake(source) = message {
